(*********************************************************************)
(*                         Diy                                       *)
(*                                                                   *)
(*   Jade Alglave, Luc Maranget INRIA Paris-Rocquencourt, France.    *)
(*                                                                   *)
(*  Copyright 2011 Institut National de Recherche en Informatique et *)
(*  en Automatique. All rights reserved. This file is distributed    *)
(*  under the terms of the Lesser GNU General Public License.        *)
(*********************************************************************)
module Make(V:Constant.S) =
  struct
    include ARMBase
    module V = V

(**********)
(* Fences *)
(**********)

    type fence = DMB

    let strong = DMB

    let pp_fence = function
      | DMB -> "DMB"

    let sig_of_fence = function
      | DMB -> 'D'


    let fold_cumul_fences f r = f DMB r
    let fold_all_fences f r = fold_cumul_fences f r

    open Code

    let orders f d1 d2 =true

(********)
(* Deps *)
(********)

    type dp = ADDR | DATA | CTRL

    let pp_dp = function
      | ADDR -> "Addr"
      | DATA -> "Data"
      | CTRL -> "Ctrl"


    let sig_of_dp = function
      | ADDR -> 'A'
      | DATA -> 'D'
      | CTRL -> 'C'


    let fold_dpr f r =  f ADDR (f CTRL r)
    let fold_dpw f r =  f ADDR (f DATA (f CTRL r))

    let ddr_default = Some ADDR
    let ddw_default = Some ADDR
    let ctrlr_default = Some CTRL
    let ctrlw_default = Some CTRL


    include
        ArchExtra.Make
        (struct
          type arch_reg = reg

          let is_symbolic = function
            | Symbolic_reg _ -> true
            | _ -> false

          let pp_reg = pp_reg
          let free_registers = allowed_for_symb
        end)
  end
