(*********************************************************************)
(*                         Diy                                       *)
(*                                                                   *)
(*   Jade Alglave, Luc Maranget INRIA Paris-Rocquencourt, France.    *)
(*                                                                   *)
(*  Copyright 2010 Institut National de Recherche en Informatique et *)
(*  en Automatique. All rights reserved. This file is distributed    *)
(*  under the terms of the Lesser GNU General Public License.        *)
(*********************************************************************)

module Make(V:Constant.S) =
  struct
    include PPCBase
    module V = V

(**********)
(* Fences *)
(**********)

    type fence = Sync | LwSync | ISync

    let strong = Sync

    let pp_fence = function
      | Sync -> "Sync"
      | LwSync -> "LwSync"
      | ISync -> "ISync"

    let sig_of_fence = function
      | Sync -> 'S'
      | LwSync -> 'L'
      | ISync -> 'I'


    let fold_cumul_fences f r = f Sync (f LwSync r)
    let fold_all_fences f r = f ISync (fold_cumul_fences f r)

    open Code

    let orders f d1 d2 = match f,d1,d2 with
    | Sync,_,_ -> true
    | LwSync,W,R -> false
    | LwSync,_,_ -> true
    | ISync,_,_ -> false


(********)
(* Deps *)
(********)

    type dp = ADDR | DATA | CTRL | CTRLISYNC

    let pp_dp = function
      | ADDR -> "Addr"
      | DATA -> "Data"
      | CTRL -> "Ctrl"
      | CTRLISYNC -> "CtrlIsync"

    let sig_of_dp = function
      | ADDR -> 'A'
      | DATA -> 'D'
      | CTRL -> 'C'
      | CTRLISYNC -> 'I'

    let fold_dpr f r =  f ADDR (f CTRL (f CTRLISYNC r))
    let fold_dpw f r =  f ADDR (f DATA (f CTRL (f CTRLISYNC r)))

    let ddr_default = Some ADDR
    let ddw_default = Some ADDR
    let ctrlr_default = Some CTRLISYNC
    let ctrlw_default = Some CTRL


    include
        ArchExtra.Make
        (struct
          type arch_reg = reg
          let is_symbolic = function
            | Symbolic_reg _ -> true
            | _ -> false
          let pp_reg = pp_reg
          let free_registers = allowed_for_symb
        end)
  end
